@@ -1,5 +1,5 @@
 /*
-    htpdate v1.3.3
+    htpdate v1.3.4
 
     Eddy Vervest <eddy@vervest.org>
     http://www.vervest.org/htp
@@ -51,6 +51,7 @@
 #include <limits.h>
 #include <pwd.h>
 #include <grp.h>
+#include <float.h>
 
 #if defined __NetBSD__ || defined __FreeBSD__ || defined __APPLE__
 #define adjtimex ntp_adjtime
@@ -60,7 +61,7 @@
 #include <openssl/ssl.h>
 #endif
 
-#define VERSION                  "1.3.3"
+#define VERSION                  "1.3.4"
 #define MAX_HTTP_HOSTS           16                /* 16 web servers */
 #define DEFAULT_HTTP_PORT        "80"
 #define DEFAULT_PROXY_PORT       "8080"
@@ -68,7 +69,7 @@
 #define DEFAULT_HTTP_VERSION     "1"               /* HTTP/1.1 */
 #define DEFAULT_TIME_LIMIT       31536000          /* 1 year */
 #define NO_TIME_LIMIT            -1
-#define ERR_TIMESTAMP            LONG_MAX          /* Err fetching date in getHTTPdate */
+#define ERR_TIMESTAMP            DBL_MAX          /* Err fetching date in getHTTPdate */
 #define DEFAULT_PRECISION        4                 /* 4 request per host */
 #define DEFAULT_MIN_SLEEP        1800              /* 30 minutes */
 #define DEFAULT_MAX_SLEEP        115200            /* 32 hours */
@@ -89,7 +90,7 @@ static int verifycert = 0;
 
 
 /* Insertion sort is more efficient (and smaller) than qsort for small lists */
-static void insertsort(double a[], long length) {
+static void insertsort(double a[], int length) {
     long i, j;
 
     for (i = 1; i < length; i++) {
@@ -193,8 +194,8 @@ static void swgid(int id) {
 }
 
 
-static long getoffset(char remote_time[25]) {
-    struct timeval  timevalue = {LONG_MAX, 0};
+static long long getoffset(char remote_time[25]) {
+    struct timeval  timevalue = {0, 0};
     struct timespec now;
     struct tm       tm;
 
@@ -307,7 +308,7 @@ static double getHTTPdate(
     /* Was the hostname and service resolvable? */
     if (rc) {
         printlog(1, "%s host or service unavailable", host);
-        return(0);                  /* Assume correct time */
+        return(ERR_TIMESTAMP);
     }
 
     /* Build a combined HTTP/1.0 and 1.1 HEAD request
@@ -345,7 +346,7 @@ static double getHTTPdate(
 
     if (rc) {
         printlog(1, "%s connection failed", host);
-        return((double)ERR_TIMESTAMP);
+        return(ERR_TIMESTAMP);
     }
 
     #ifdef ENABLE_HTTPS
@@ -359,25 +360,22 @@ static double getHTTPdate(
             rc = proxyCONNECT(server_s, host, port, proxy, proxyport, httpversion);
             if (rc != 1) {
                 printlog(1, "Proxy error: %i", rc);
+                return(ERR_TIMESTAMP);
             }
         }
-        rc = SSL_set_fd(conn, server_s);
-        if (rc != 1) {
-            printlog(1, "SSL error1: %i", rc);
-            rc = -1;
+        if (! SSL_set_fd(conn, server_s)) {
+            printlog(1, "TLS error1");
+            return(ERR_TIMESTAMP);
         } else {
-           rc = SSL_connect(conn);
-           if (rc != 1) {
-               printlog(1, "SSL error2: %i", rc);
-               rc = -1;
+           if (SSL_connect(conn) != 1) {
+               printlog(1, "TLS error2");
+               return(ERR_TIMESTAMP);
            }
         }
     }
     #endif
 
-    long offset = 0;
-    long first_offset = 0;
-    long prev_offset = 0;
+    long long offset = 0, first_offset = 0, prev_offset = 0;
     long nap = 1e9L;
     long when = nap >> precision;
     do {
@@ -440,6 +438,7 @@ static double getHTTPdate(
                     remote_time, rtt / (long)1e6, offset);
             } else {
                 printlog(1, "%s no timestamp", host);
+                return(ERR_TIMESTAMP);
             }
         }                           /* bytes received */
         precision--;
@@ -456,7 +455,6 @@ static double getHTTPdate(
     /* Rounding */
     if (debug) printlog(0, "when: %ld, nap: %ld", when, nap);
     if (when + nap == 1e9 && offset == 0) return 0;
-    if (offset < 0) when -= nap;
 
     /* Return the time delta between web server time (timevalue)
        and system time (now)
@@ -522,15 +520,16 @@ static int init_frequency(char *driftfile) {
     fp = fopen(driftfile, "r");
     if (fp != NULL) {
         if (fscanf(fp, "%li", &tmx.freq)) {
-           printlog(0, "Reading frequency from file %li", tmx.freq);
+            printlog(0, "Reading frequency from file %li", tmx.freq);
+            fclose(fp);
         } else {
             printlog(1, "Error reading frequency from %s", driftfile);
+            fclose(fp);
+            return -1;
         }
-        fclose(fp);
-    return -1;
     } else {
         printlog(1, "Error reading frequency from %s", driftfile);
-    return -1;
+        return -1;
     }
 
     if ((tmx.freq < -MAX_DRIFT) || (tmx.freq > MAX_DRIFT))
@@ -680,7 +679,6 @@ static void runasdaemon(char *pidfile) {
 int main(int argc, char *argv[]) {
     char            *host = NULL, *proxy = NULL, *proxyport = NULL;
     char            *port = NULL;
-    char            *hostport = NULL;
     char            *path = NULL;
     char            *scheme = NULL;
     char            *httpversion = DEFAULT_HTTP_VERSION;
@@ -688,14 +686,14 @@ int main(int argc, char *argv[]) {
     char            *user = NULL, *userstr = NULL, *group = NULL;
     double          timeavg, drift = 0;
     double          timedelta[MAX_HTTP_HOSTS-1];
-    long            numservers;
-    long            precision = DEFAULT_PRECISION;
+    int             numservers;
+    int             precision = DEFAULT_PRECISION;
     int             setmode = 0;
     int             i, param;
     int             daemonize = 0, foreground = 0;
     int             noproxyenv = 0;
     int             ipversion = DEFAULT_IP_VERSION;
-    long            timelimit = DEFAULT_TIME_LIMIT;
+    long long       timelimit = DEFAULT_TIME_LIMIT;
     int             minsleep = DEFAULT_MIN_SLEEP;
     int             maxsleep = DEFAULT_MAX_SLEEP;
     int             sleeptime = minsleep;
@@ -877,30 +875,29 @@ int main(int argc, char *argv[]) {
         /* Initialize number of received valid timestamps, good timestamps
            and the average of the good timestamps
         */
-        long   validtimes = 0, goodtimes = 0;
+        int    validtimes = 0, goodtimes = 0;
         double sumtimes = 0, mean = 0;
 
         /* Loop through the time sources (web servers); poll cycle */
         for (i = optind; i < argc; i++) {
 
             /* host:port is stored in argv[i] */
-            hostport = (char *)argv[i];
-            host = strdup(hostport);
+            char *hostport = strdup(argv[i]);
+            host = hostport;
             splitURL(&scheme, &host, &port, &path);
 
             double offset = getHTTPdate(scheme, host, port, path,
                 proxy, proxyport, httpversion, ipversion, precision);
-            if (debug) printlog(0, "offset: %.6f", offset);
+            free(hostport);
+            if (debug && offset != ERR_TIMESTAMP) {
+                printlog(0, "offset: %.6f", offset);
+            }
 
             /* Only include valid responses in timedelta[] */
-            if (timelimit == NO_TIME_LIMIT || (offset < timelimit && offset > -timelimit)) {
+            if ((timelimit == NO_TIME_LIMIT && offset != ERR_TIMESTAMP) || (offset < timelimit && offset > -timelimit)) {
                 timedelta[validtimes] = offset;
                 validtimes++;
             }
-
-            /* Sleep for a while, unless a time offset is detected */
-            if ((daemonize || foreground) && !offset)
-                sleep(sleeptime / numservers);
         }
 
         /* Sort the timedelta results */
@@ -966,20 +963,23 @@ int main(int argc, char *argv[]) {
                 /* Increase polling interval */
                 if (sleeptime < maxsleep) sleeptime <<= 1;
             }
-            if (daemonize || foreground)
-                printlog(0, "poll: %ld s", sleeptime);
 
-            } else {
-                printlog(1, "No server suitable for synchronization found");
-                /* Sleep for minsleep to avoid flooding */
-                if (daemonize || foreground)
-                    sleep(minsleep);
-                else
-                    exit(1);
+            if (daemonize || foreground) {
+                printlog(0, "sleep for %ld s", sleeptime);
+                sleep(sleeptime);
             }
 
-            /* After first poll cycle do not step through time, only adjust */
-            if (setmode != 3) setmode = 1;
+        } else {
+            printlog(1, "No server suitable for synchronization found");
+            /* Sleep for minsleep to avoid flooding */
+            if (daemonize || foreground)
+                sleep(minsleep);
+            else
+                exit(1);
+        }
+
+        /* After first poll cycle do not step through time, only adjust */
+        if (setmode != 3) setmode = 1;
 
     } while (daemonize || foreground);         /* end of infinite while loop */
 
